package com.plannaplan.responses.mappers;

import static org.junit.Assert.assertTrue;

import java.lang.reflect.Field;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;

import com.plannaplan.entities.Groups;
import com.plannaplan.entities.Lecturer;
import com.plannaplan.responses.models.GroupDefaultResponse;
import com.plannaplan.responses.models.CourseWithGroupsResponse;
import com.plannaplan.responses.models.GroupWithCapacityResponse;
import com.plannaplan.types.WeekDay;

import org.junit.Test;

public class GroupsMappersTest {
        @Test
        public void shouldMapToResponseIncludingCapacity() {
                final List<Groups> gropus = Arrays.asList(new Groups(42, "A4-1", null, 520, WeekDay.MONDAY,
                                new Lecturer("krul.", "Wladyslaw", "Potocki")));
                final List<GroupWithCapacityResponse> response = GroupsMappers.mapToCapacityResponse(gropus);

                assert (response.get(0).getCapacity() == 42);
                assert (response.get(0) instanceof GroupWithCapacityResponse);
                assert (response.size() == 1);
        }

        @Test
        public void shouldMapToResponseWiothoutCapacity() {
                final List<Groups> gropus = Arrays.asList(new Groups(42, "A4-1", null, 520, WeekDay.MONDAY,
                                new Lecturer("krul.", "Wladyslaw", "Potocki")));
                final List<GroupDefaultResponse> response = GroupsMappers.mapToDefaultResponse(gropus);

                assert (response.get(0) instanceof GroupDefaultResponse);
                assert (response.size() == 1);
        }

        @Test
        public void shouldMapToGetCourseGroupsResponse() {
                final List<Groups> groups = Arrays.asList(
                                new Groups(150, "A4-1", null, 520, WeekDay.MONDAY,
                                                new Lecturer("krul.", "Wladyslaw", "Potocki")),
                                new Groups(24, "A4-1", null, 520, WeekDay.MONDAY,
                                                new Lecturer("krul.", "Wladyslaw", "Potocki")));

                final CourseWithGroupsResponse<GroupDefaultResponse> response = GroupsMappers
                                .mapToGetCourseGroupsDefaultResponse(groups);

                assertTrue(response.getClasses().size() == 1);
                assertTrue(response.getLectures().size() == 1);

        }

        @Test
        public void shouldMapToGetCourseGroupsResponseWithTakenPlaces() throws NoSuchFieldException, SecurityException,
                        IllegalArgumentException, IllegalAccessException {
                final Field reader = Groups.class.getDeclaredField("id");
                reader.setAccessible(true);

                final List<Groups> groups = Arrays.asList(
                                new Groups(150, "A4-1", null, 520, WeekDay.MONDAY,
                                                new Lecturer("krul.", "Wladyslaw", "Potocki")),
                                new Groups(24, "A4-1", null, 520, WeekDay.MONDAY,
                                                new Lecturer("krul.", "Wladyslaw", "Potocki")));
                reader.set(groups.get(0), Long.valueOf(0));
                reader.set(groups.get(1), Long.valueOf(1));

                final HashMap<Long, Integer> placeMap = new HashMap<>();
                placeMap.put(Long.valueOf(0), 0);
                placeMap.put(Long.valueOf(1), 21);

                final CourseWithGroupsResponse<GroupDefaultResponse> response = GroupsMappers
                                .mapToGetCourseGroupsDefaultResponse(groups, placeMap);

                assertTrue(response.getClasses().size() == 1);
                assertTrue(response.getLectures().size() == 1);
                assertTrue(response.getLectures().get(0).getTakenPlaces() == 0);
                assertTrue(response.getClasses().get(0).getTakenPlaces() == 21);
        }

        @Test
        public void shouldMapToDefaultResponseWithTakenPlace() throws NoSuchFieldException, SecurityException,
                        IllegalArgumentException, IllegalAccessException {
                final Field reader = Groups.class.getDeclaredField("id");
                reader.setAccessible(true);

                final Groups group1 = new Groups(150, "A4-1", null, 520, WeekDay.MONDAY,
                                new Lecturer("krul.", "Wladyslaw", "Potocki"));
                final Groups group2 = new Groups(24, "A4-1", null, 520, WeekDay.MONDAY,
                                new Lecturer("krul.", "Wladyslaw", "Potocki"));

                reader.set(group1, Long.valueOf(0));
                reader.set(group2, Long.valueOf(1));

                final HashMap<Long, Integer> placeMap = new HashMap<>();
                placeMap.put(Long.valueOf(0), 5);
                placeMap.put(Long.valueOf(1), 56);

                final List<GroupDefaultResponse> response = GroupsMappers.mapToDefaultResponse(List.of(group1, group2),
                                placeMap);

                assertTrue(response.size() == 2);
                assertTrue(response.get(0).getTakenPlaces() == 5);
                assertTrue(response.get(1).getTakenPlaces() == 56);
        }

        @Test
        public void shouldMapToCapacityResponseWithTakenPlace() throws NoSuchFieldException, SecurityException,
                        IllegalArgumentException, IllegalAccessException {
                final Field reader = Groups.class.getDeclaredField("id");
                reader.setAccessible(true);

                final Groups group1 = new Groups(150, "A4-1", null, 520, WeekDay.MONDAY,
                                new Lecturer("krul.", "Wladyslaw", "Potocki"));
                final Groups group2 = new Groups(24, "A4-1", null, 520, WeekDay.MONDAY,
                                new Lecturer("krul.", "Wladyslaw", "Potocki"));

                reader.set(group1, Long.valueOf(0));
                reader.set(group2, Long.valueOf(1));

                final HashMap<Long, Integer> placeMap = new HashMap<>();
                placeMap.put(Long.valueOf(0), 5);
                placeMap.put(Long.valueOf(1), 56);

                final List<GroupWithCapacityResponse> response = GroupsMappers
                                .mapToCapacityResponse(List.of(group1, group2), placeMap);

                assertTrue(response.size() == 2);
                assertTrue(response.get(0).getTakenPlaces() == 5);
                assertTrue(response.get(0).getCapacity() == 150);
                assertTrue(response.get(1).getTakenPlaces() == 56);
                assertTrue(response.get(1).getCapacity() == 24);
        }

}
